@@ -54,9 +54,8 @@ class Data(TemplateMixin, IOMixin, CallableTemplateMixin):
             name=self.name,
             node_selector=self.node_selector,
             plugin=self.plugin,
-            priority=self.priority,
             priority_class_name=self.priority_class_name,
-            retry_strategy=self.retry_strategy,
+            retry_strategy=self._build_retry_strategy(),
             scheduler_name=self.scheduler_name,
             security_context=self.pod_security_context,
             service_account_name=self.service_account_name,
