@@ -6,12 +6,33 @@ Input/Output are only defined in this file if Pydantic v2 is installed.
 from typing import Any
 
 from hera.shared._pydantic import _PYDANTIC_VERSION
+from hera.workflows._context import _context
 from hera.workflows.io._io_mixins import InputMixin, OutputMixin
 
 if _PYDANTIC_VERSION == 2:
     from pydantic import BaseModel
 
-    class Input(InputMixin, BaseModel):  # type: ignore[misc]
+    class _DeclaringEnabledModel(BaseModel):
+        def __new__(cls, **kwargs):
+            if _context.declaring:
+                # Intercept the declaration to avoid validation on the templated strings
+                # We must then turn off declaring mode to be able to "construct" an instance
+                # of the InputMixin subclass.
+                _context.declaring = False
+                instance = cls.construct(**kwargs)
+                _context.declaring = True
+                return instance
+            else:
+                return super(_DeclaringEnabledModel, cls).__new__(cls)
+
+        def __init__(self, /, **kwargs):
+            if _context.declaring:
+                # Return in order to skip validation of `construct`ed instance
+                return
+
+            super().__init__(**kwargs)
+
+    class Input(InputMixin, _DeclaringEnabledModel):
         """Input model usable by the Hera Runner.
 
         Input is a Pydantic model which users can create a subclass of. When a subclass
@@ -20,7 +41,7 @@ if _PYDANTIC_VERSION == 2:
         for the script_pydantic_io experimental feature.
         """
 
-    class Output(OutputMixin, BaseModel):  # type: ignore[misc]
+    class Output(OutputMixin, _DeclaringEnabledModel):
         """Output model usable by the Hera Runner.
 
         Output is a Pydantic model which users can create a subclass of. When a subclass
