@@ -11,6 +11,7 @@ import sys
 import textwrap
 from abc import abstractmethod
 from functools import wraps
+from pathlib import Path
 from typing import (
     Any,
     Callable,
@@ -47,8 +48,10 @@ from hera.shared._pydantic import _PYDANTIC_VERSION, root_validator, validator
 from hera.shared._type_util import (
     construct_io_from_annotation,
     get_workflow_annotation,
+    is_annotated,
     is_subscripted,
     origin_type_issupertype,
+    unwrap_annotation,
 )
 from hera.shared.serialization import serialize
 from hera.workflows._context import _context
@@ -214,9 +217,8 @@ class Script(
                 parallelism=self.parallelism,
                 plugin=self.plugin,
                 pod_spec_patch=self.pod_spec_patch,
-                priority=self.priority,
                 priority_class_name=self.priority_class_name,
-                retry_strategy=self.retry_strategy,
+                retry_strategy=self._build_retry_strategy(),
                 scheduler_name=self.scheduler_name,
                 script=self._build_script(),
                 security_context=self.pod_security_context,
@@ -443,7 +445,17 @@ def _get_inputs_from_callable(source: Callable) -> Tuple[List[Parameter], List[A
     artifacts = []
 
     for func_param in inspect.signature(source).parameters.values():
-        if not is_subscripted(func_param.annotation) and issubclass(func_param.annotation, (InputV1, InputV2)):
+        # If the annotation is not subscripted, then we can directly check if it is an Input type annotation.
+        # Otherwise, we check if it is of the form `Annotated[...]`, and subsequently check whether the unwrapped
+        # annotation is a class which we can then check if it is a subclass. Otherwise, an annotation of the form
+        # `Annotated[Literal[...]]` will raise an exception as `typing.Literal` is not a class (but an object).
+        if (
+            not is_subscripted(func_param.annotation)
+            and issubclass(func_param.annotation, (InputV1, InputV2))
+            or is_annotated(func_param.annotation)
+            and inspect.isclass(unwrap_annotation(func_param.annotation))
+            and issubclass(unwrap_annotation(func_param.annotation), (InputV1, InputV2))
+        ):
             if not _flag_enabled(_SCRIPT_PYDANTIC_IO_FLAG):
                 raise ValueError(
                     (
@@ -597,9 +609,9 @@ class _ScriptDecoratedFunction(Generic[FuncIns, FuncRCov], Protocol):
         continue_on: Optional[ContinueOn] = ...,
         hooks: Optional[Dict[str, LifecycleHook]] = ...,
         on_exit: Optional[Union[str, Templatable]] = ...,
-        template: Optional[Union[str, _ModelTemplate, TemplateMixin, CallableTemplateMixin]] = ...,
+        template: Optional[Union[str, _ModelTemplate, Templatable]] = ...,
         template_ref: Optional[TemplateRef] = ...,
-        inline: Optional[Union[_ModelTemplate, TemplateMixin]] = ...,
+        inline: Optional[Union[_ModelTemplate, Templatable]] = ...,
         when: Optional[str] = ...,
         with_sequence: Optional[_ModelSequence] = ...,
         arguments: ArgumentsT = ...,
@@ -625,9 +637,9 @@ class _ScriptDecoratedFunction(Generic[FuncIns, FuncRCov], Protocol):
         continue_on: Optional[ContinueOn] = ...,
         hooks: Optional[Dict[str, LifecycleHook]] = ...,
         on_exit: Optional[Union[str, Templatable]] = ...,
-        template: Optional[Union[str, _ModelTemplate, TemplateMixin, CallableTemplateMixin]] = ...,
+        template: Optional[Union[str, _ModelTemplate, Templatable]] = ...,
         template_ref: Optional[TemplateRef] = ...,
-        inline: Optional[Union[_ModelTemplate, TemplateMixin]] = ...,
+        inline: Optional[Union[_ModelTemplate, Templatable]] = ...,
         when: Optional[str] = ...,
         with_sequence: Optional[_ModelSequence] = ...,
         arguments: ArgumentsT = ...,
@@ -852,11 +864,19 @@ class RunnerScriptConstructor(ScriptConstructor):
 
         if values.get("args") is not None:
             raise ValueError("Cannot specify args when callable is True")
+
+        module = values["source"].__module__
+
+        if module == "__main__":
+            from hera.workflows._runner.util import create_module_string
+
+            module = create_module_string(Path(values["source"].__globals__["__file__"]))
+
         values["args"] = [
             "-m",
             "hera.workflows.runner",
             "-e",
-            f"{values['source'].__module__}:{values['source'].__name__}",
+            f"{module}:{values['source'].__name__}",
         ]
 
         return values
