@@ -4,10 +4,11 @@ See <https://argoproj.github.io/argo-workflows/workflow-concepts/#the-workflow>
 for more on Workflows.
 """
 
+import asyncio
 import logging
 import time
 from pathlib import Path
-from typing import Annotated, Any, Callable, Dict, List, Optional, Type, TypeVar, Union, get_args
+from typing import Annotated, Any, Callable, Dict, List, Optional, Type, TypeVar, Union
 
 from typing_extensions import ParamSpec
 
@@ -23,6 +24,7 @@ from hera.workflows._mixins import (
     VolumeMixin,
     VolumesT,
 )
+from hera.workflows.async_service import AsyncWorkflowsService
 from hera.workflows.exceptions import InvalidType
 from hera.workflows.models import (
     Affinity,
@@ -41,7 +43,7 @@ from hera.workflows.models import (
     PodDNSConfig,
     PodGC,
     PodSecurityContext,
-    RetryStrategy,
+    RetryStrategy as ModelRetryStrategy,
     Synchronization,
     Template as _ModelTemplate,
     Time,
@@ -57,7 +59,8 @@ from hera.workflows.models import (
     WorkflowTemplateRef,
 )
 from hera.workflows.parameter import Parameter
-from hera.workflows.protocol import Templatable, TTemplate, TWorkflow, VolumeClaimable
+from hera.workflows.protocol import Templatable, TWorkflow, VolumeClaimable
+from hera.workflows.retry_strategy import RetryStrategy
 from hera.workflows.service import WorkflowsService
 from hera.workflows.template_set import TemplateSet
 from hera.workflows.workflow_status import WorkflowStatus
@@ -103,22 +106,31 @@ class Workflow(
             return self.on_exit._build_template().name  # type: ignore
         return self.on_exit
 
-    def _build_templates(self) -> Optional[List[TTemplate]]:
+    def _build_retry_strategy(self) -> Optional[ModelRetryStrategy]:
+        if self.retry_strategy is None:
+            return None
+
+        if isinstance(self.retry_strategy, RetryStrategy):
+            return self.retry_strategy.build()
+
+        return self.retry_strategy
+
+    def _build_templates(self) -> Optional[List[_ModelTemplate]]:
         """Builds the templates into an Argo schema."""
-        templates = []
+        templates: List[_ModelTemplate] = []
         for template in self.templates:
             if isinstance(template, HookMixin):
                 template = template._dispatch_hooks()
 
             if isinstance(template, Templatable):
                 templates.append(template._build_template())
-            elif isinstance(template, get_args(TTemplate)):
+            elif isinstance(template, _ModelTemplate):
                 templates.append(template)
             else:
                 raise InvalidType(f"{type(template)} is not a valid template type")
 
             if isinstance(template, VolumeClaimable):
-                claims = template._build_persistent_volume_claims()  # type: ignore
+                claims = template._build_persistent_volume_claims()
                 # If there are no claims, continue, nothing to add
                 if not claims:
                     continue
@@ -202,11 +214,13 @@ class Workflow(
     ] = None
     pod_gc: Annotated[Optional[PodGC], _WorkflowModelMapper("spec.pod_gc")] = None
     pod_metadata: Annotated[Optional[Metadata], _WorkflowModelMapper("spec.pod_metadata")] = None
-    pod_priority: Annotated[Optional[int], _WorkflowModelMapper("spec.pod_priority")] = None
     pod_priority_class_name: Annotated[Optional[str], _WorkflowModelMapper("spec.pod_priority_class_name")] = None
     pod_spec_patch: Annotated[Optional[str], _WorkflowModelMapper("spec.pod_spec_patch")] = None
     priority: Annotated[Optional[int], _WorkflowModelMapper("spec.priority")] = None
-    retry_strategy: Annotated[Optional[RetryStrategy], _WorkflowModelMapper("spec.retry_strategy")] = None
+    retry_strategy: Annotated[
+        Optional[Union[RetryStrategy, ModelRetryStrategy]],
+        _WorkflowModelMapper("spec.retry_strategy", _build_retry_strategy),
+    ] = None
     scheduler_name: Annotated[Optional[str], _WorkflowModelMapper("spec.scheduler_name")] = None
     security_context: Annotated[Optional[PodSecurityContext], _WorkflowModelMapper("spec.security_context")] = None
     service_account_name: Annotated[Optional[str], _WorkflowModelMapper("spec.service_account_name")] = None
@@ -241,7 +255,11 @@ class Workflow(
     volumes: Annotated[VolumesT, _WorkflowModelMapper("spec.volumes", VolumeMixin._build_volumes)] = None
 
     # Hera-specific fields
-    workflows_service: Optional[WorkflowsService] = None
+    workflows_service: Optional[Union[WorkflowsService, AsyncWorkflowsService]] = None
+
+    pod_priority: Optional[int] = None
+    """DEPRECATED: The spec.podPriority field was removed in 3.7, so does not map to
+       anything. Use pod_priority_class_name instead."""
 
     @validator("name", pre=True, always=True)
     def _set_name(cls, v):
@@ -388,12 +406,13 @@ class Workflow(
         Parameters
         ----------
         wait: bool = False
-            If false then the workflow is created asynchronously and the function returns immediately.
+            If false then the workflow is created and the function returns immediately after the server
+            creates the Workflow.
             If true then the workflow is created and the function blocks until the workflow is done executing.
         poll_interval: int = 5
             The interval in seconds to poll the workflow status if wait is true. Ignored when wait is false.
         """
-        assert self.workflows_service, "workflow service not initialized"
+        assert isinstance(self.workflows_service, WorkflowsService), "workflows service not initialized"
         assert self.namespace, "workflow namespace not defined"
 
         wf = self.workflows_service.create_workflow(
@@ -416,7 +435,7 @@ class Workflow(
         poll_interval: int = 5
             The interval in seconds to poll the workflow status.
         """
-        assert self.workflows_service is not None, "workflow service not initialized"
+        assert isinstance(self.workflows_service, WorkflowsService), "workflows service not initialized"
         assert self.namespace is not None, "workflow namespace not defined"
         assert self.name is not None, "workflow name not defined"
 
@@ -438,6 +457,7 @@ class Workflow(
         while status == WorkflowStatus.running:
             time.sleep(poll_interval)
             wf = self.workflows_service.get_workflow(wf.metadata.name, namespace=self.namespace)
+            assert wf.metadata.name is not None
             assert wf.status is not None, f"workflow status not defined for workflow {wf.metadata.name}"
             assert wf.status.phase is not None, f"workflow phase not defined for workflow status {wf.status}"
             status = WorkflowStatus.from_argo_status(wf.status.phase)
@@ -445,13 +465,84 @@ class Workflow(
 
     def lint(self) -> TWorkflow:
         """Lints the Workflow using the Argo cluster."""
-        assert self.workflows_service, "workflow service not initialized"
+        assert isinstance(self.workflows_service, WorkflowsService), "workflows service not initialized"
         assert self.namespace, "workflow namespace not defined"
         return self.workflows_service.lint_workflow(
             WorkflowLintRequest(workflow=self.build()),  # type: ignore
             namespace=self.namespace,
         )
 
+    async def async_create(self, wait: bool = True, poll_interval: int = 5) -> TWorkflow:
+        """Creates the Workflow on the Argo cluster. Note that `wait` is `True` by default as this is an async function.
+
+        Parameters
+        ----------
+        wait: bool = True
+            If false then the workflow is created and the function returns immediately.
+            If true then the workflow is created and the function blocks until the workflow is done executing.
+        poll_interval: int = 5
+            The interval in seconds to poll the workflow status if wait is true. Ignored when wait is false.
+        """
+        assert isinstance(self.workflows_service, AsyncWorkflowsService), "workflows service not initialized"
+        assert self.namespace, "workflow namespace not defined"
+
+        wf = await self.workflows_service.create_workflow(
+            WorkflowCreateRequest(workflow=self.build()),  # type: ignore
+            namespace=self.namespace,
+        )
+        # set the workflow name to the name returned by the API, which helps cover the case of users relying on
+        # `generate_name=True`
+        self.name = wf.metadata.name
+
+        if wait:
+            return await self.async_wait(poll_interval=poll_interval)
+        return wf
+
+    async def async_wait(self, poll_interval: int = 5) -> TWorkflow:
+        """Waits for the Workflow to complete execution.
+
+        Parameters
+        ----------
+        poll_interval: int = 5
+            The interval in seconds to poll the workflow status.
+        """
+        assert isinstance(self.workflows_service, AsyncWorkflowsService), "workflows service not initialized"
+        assert self.namespace is not None, "workflow namespace not defined"
+        assert self.name is not None, "workflow name not defined"
+
+        # here we use the sleep interval to wait for the workflow post creation. This is to address a potential
+        # race conditions such as:
+        # 1. Argo server says "workflow was accepted" but the workflow is not yet created
+        # 2. Hera wants to verify the status of the workflow, but it's not yet defined because it's not created
+        # 3. Argo finally creates the workflow
+        # 4. Hera throws an `AssertionError` because the phase assertion fails
+        await asyncio.sleep(poll_interval)
+        wf = await self.workflows_service.get_workflow(self.name, namespace=self.namespace)
+        assert wf.metadata.name is not None, f"workflow name not defined for workflow {self.name}"
+
+        assert wf.status is not None, f"workflow status not defined for workflow {wf.metadata.name}"
+        assert wf.status.phase is not None, f"workflow phase not defined for workflow status {wf.status}"
+        status = WorkflowStatus.from_argo_status(wf.status.phase)
+
+        # keep polling for workflow status until completed, at the interval dictated by the user
+        while status == WorkflowStatus.running:
+            await asyncio.sleep(poll_interval)
+            wf = await self.workflows_service.get_workflow(wf.metadata.name, namespace=self.namespace)
+            assert wf.metadata.name is not None
+            assert wf.status is not None, f"workflow status not defined for workflow {wf.metadata.name}"
+            assert wf.status.phase is not None, f"workflow phase not defined for workflow status {wf.status}"
+            status = WorkflowStatus.from_argo_status(wf.status.phase)
+        return wf
+
+    async def async_lint(self) -> TWorkflow:
+        """Lints the Workflow using the Argo cluster."""
+        assert isinstance(self.workflows_service, AsyncWorkflowsService), "workflows service not initialized"
+        assert self.namespace, "workflow namespace not defined"
+        return await self.workflows_service.lint_workflow(
+            WorkflowLintRequest(workflow=self.build()),  # type: ignore
+            namespace=self.namespace,
+        )
+
     def _add_sub(self, node: Any):
         """Adds the given node (expected to satisfy the `Templatable` protocol) to the context."""
         if not isinstance(node, (Templatable, _ModelTemplate)):
